@@ -13,7 +13,7 @@ from qtpy.QtCore import Qt
 
 from glue.core.application_base import Application
 from glue.core.message import ApplicationClosedMessage, DataCollectionMessage
-from glue.core import command, Data
+from glue.core import command, BaseData
 from glue.core.coordinates import WCSCoordinates
 from glue import env
 from glue.main import load_plugins
@@ -69,7 +69,7 @@ def _fix_ipython_pylab():
             pass
 
     try:
-        shell.enable_pylab('inline', import_all=True)
+        shell.enable_pylab('agg', import_all=True)
     except ValueError:
         # if the shell is a normal terminal shell, we get here
         pass
@@ -78,6 +78,9 @@ def _fix_ipython_pylab():
         pass
     except UsageError:
         pass
+    except KeyError:
+        # old versions of ipython
+        pass
 
     # Make sure we disable interactive mode (where figures get redrawn for
     # every single Matplotlib command)
@@ -246,7 +249,7 @@ class ImportHelper(object):
                 raise TypeError("Data loader should return list of "
                                 "Data objects")
             for item in data:
-                if not isinstance(item, Data):
+                if not isinstance(item, BaseData):
                     raise TypeError("Data loader should return list of "
                                     "Data objects")
             app.add_datasets(app.data_collection, data)
@@ -368,12 +371,27 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
         # Data toolbar
 
-        self._data_toolbar = QtWidgets.QToolBar()
+        self._data_toolbar = QtWidgets.QToolBar("Session and Data toolbar")
 
         self._data_toolbar.setIconSize(QtCore.QSize(16, 16))
 
+        self._button_open_session = QtWidgets.QToolButton()
+        self._button_open_session.setText("Open Session")
+        self._button_open_session.setIcon(get_icon('glue_open'))
+        self._button_open_session.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        self._button_open_session.clicked.connect(self._restore_session)
+
+        self._data_toolbar.addWidget(self._button_open_session)
+
+        self._button_save_session = QtWidgets.QToolButton()
+        self._button_save_session.setText("Export Session")
+        self._button_save_session.setIcon(get_icon('glue_filesave'))
+        self._button_save_session.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        self._button_save_session.clicked.connect(self._choose_save_session)
+
+        self._data_toolbar.addWidget(self._button_save_session)
         self._button_open_data = QtWidgets.QToolButton()
-        self._button_open_data.setText("Open Data")
+        self._button_open_data.setText("Import Data")
         self._button_open_data.setIcon(get_icon('glue_open'))
         self._button_open_data.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
         self._button_open_data.clicked.connect(self._import_helper._choose_load_data_wizard)
@@ -396,46 +414,14 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
         self._data_toolbar.addWidget(self._button_link_data)
 
-        self._button_ipython = QtWidgets.QToolButton()
-        self._button_ipython.setCheckable(True)
-        self._button_ipython.setText("IPython Terminal")
-        self._button_ipython.setIcon(get_icon('IPythonConsole'))
-        self._button_ipython.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
-        self._button_ipython.clicked.connect(self._toggle_terminal)
-
-        self._data_toolbar.addWidget(self._button_ipython)
-
-        self._button_open_session = QtWidgets.QToolButton()
-        self._button_open_session.setText("Open Session")
-        self._button_open_session.setIcon(get_icon('glue_open'))
-        self._button_open_session.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
-        self._button_open_session.clicked.connect(self._restore_session)
-
-        self._data_toolbar.addWidget(self._button_open_session)
-
-        self._button_save_session = QtWidgets.QToolButton()
-        self._button_save_session.setText("Export Session")
-        self._button_save_session.setIcon(get_icon('glue_filesave'))
-        self._button_save_session.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
-        self._button_save_session.clicked.connect(self._choose_save_session)
-
-        self._data_toolbar.addWidget(self._button_save_session)
-
         self._button_edit_components = QtWidgets.QToolButton()
-        self._button_edit_components.setText("Add/edit arithmetic attributes")
-        self._button_edit_components.setIcon(get_icon('pencil'))
+        self._button_edit_components.setText("Arithmetic attributes")
+        self._button_edit_components.setIcon(get_icon('arithmetic'))
         self._button_edit_components.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
         self._button_edit_components.clicked.connect(self._artihmetic_dialog)
 
         self._data_toolbar.addWidget(self._button_edit_components)
 
-        spacer = QtWidgets.QWidget()
-        spacer.setMinimumSize(20, 10)
-        spacer.setSizePolicy(QtWidgets.QSizePolicy.Fixed,
-                             QtWidgets.QSizePolicy.Preferred)
-
-        self._data_toolbar.addWidget(spacer)
-
         self.addToolBar(self._data_toolbar)
 
         self._on_data_collection_change()
@@ -449,9 +435,9 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
         # Error console toolbar
 
-        self._console_toolbar = QtWidgets.QToolBar()
+        self._console_toolbar = QtWidgets.QToolBar('Advanced toolbar')
 
-        self._console_toolbar.setIconSize(QtCore.QSize(16, 16))
+        self._console_toolbar.setIconSize(QtCore.QSize(14, 14))
 
         spacer = QtWidgets.QWidget()
         spacer.setSizePolicy(QtWidgets.QSizePolicy.Expanding,
@@ -459,6 +445,15 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
         self._console_toolbar.addWidget(spacer)
 
+        self._button_ipython = QtWidgets.QToolButton()
+        self._button_ipython.setCheckable(True)
+        self._button_ipython.setText("Terminal")
+        self._button_ipython.setIcon(get_icon('IPythonConsole'))
+        self._button_ipython.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
+        self._button_ipython.clicked.connect(self._toggle_terminal)
+
+        self._console_toolbar.addWidget(self._button_ipython)
+
         self._button_preferences = QtWidgets.QToolButton()
         self._button_preferences.setText("Preferences")
         self._button_preferences.setIcon(get_icon('glue_settings'))
@@ -468,7 +463,7 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         self._console_toolbar.addWidget(self._button_preferences)
 
         self._button_console = QtWidgets.QToolButton()
-        self._button_console.setText("View Error Console")
+        self._button_console.setText("Error Console")
         self._button_console.setToolButtonStyle(Qt.ToolButtonTextOnly)
 
         self._console_toolbar.addWidget(self._button_console)
@@ -578,22 +573,24 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         if self.tab_widget.count() == 1:
             return
 
-        if warn and not os.environ.get('GLUE_TESTING'):
-            buttons = QtWidgets.QMessageBox.Ok | QtWidgets.QMessageBox.Cancel
-            dialog = QtWidgets.QMessageBox.warning(
-                self, "Confirm Close",
-                "Are you sure you want to close this tab? "
-                "This will close all data viewers in the tab.",
-                buttons=buttons, defaultButton=QtWidgets.QMessageBox.Cancel)
-            if not dialog == QtWidgets.QMessageBox.Ok:
-                return
-
         w = self.tab_widget.widget(index)
 
-        for window in w.subWindowList():
-            widget = window.widget()
-            if isinstance(widget, DataViewer):
-                widget.close(warn=False)
+        if len(w.subWindowList()) > 0:
+
+            if warn and not os.environ.get('GLUE_TESTING'):
+                buttons = QtWidgets.QMessageBox.Ok | QtWidgets.QMessageBox.Cancel
+                dialog = QtWidgets.QMessageBox.warning(
+                    self, "Confirm Close",
+                    "Are you sure you want to close this tab? "
+                    "This will close all data viewers in the tab.",
+                    buttons=buttons, defaultButton=QtWidgets.QMessageBox.Cancel)
+                if not dialog == QtWidgets.QMessageBox.Ok:
+                    return
+
+            for window in w.subWindowList():
+                widget = window.widget()
+                if isinstance(widget, DataViewer):
+                    widget.close(warn=False)
 
         w.close()
 
@@ -1002,7 +999,11 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         cmd = command.NewDataViewer(viewer=client, data=data)
         return self.do(cmd)
 
-    new_data_viewer = defer_draw(Application.new_data_viewer)
+    @defer_draw
+    def new_data_viewer(self, viewer_class, data=None, state=None):
+        viewer = super(GlueApplication, self).new_data_viewer(viewer_class, data=data, state=state)
+        viewer.show()
+        return viewer
 
     def _choose_save_session(self, *args):
         """ Save the data collection and hub to file.
@@ -1013,10 +1014,11 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
         # include file filter twice, so it shows up in Dialog
         outfile, file_filter = compat.getsavefilename(
             parent=self, basedir=getattr(self, '_last_session_name', 'session.glu'),
-            filters=("Glue Session with absolute paths to data(*.glu);; "
-                     "Glue Session with relative paths to data(*.glu);; "
+            filters=("Glue Session with absolute paths to data (*.glu);; "
+                     "Glue Session with relative paths to data (*.glu);; "
                      "Glue Session including data (*.glu)"),
-            selectedfilter=getattr(self, '_last_session_filter', ''))
+            selectedfilter=getattr(self, '_last_session_filter',
+                                   'Glue Session with relative paths to data (*.glu)'))
 
         # This indicates that the user cancelled
         if not outfile:
@@ -1329,7 +1331,7 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
             self.tab_bar.setTabText(index, value)
 
     @staticmethod
-    def _choose_merge(data, others):
+    def _choose_merge(data, others, suggested_label):
 
         w = load_ui('merge.ui', None, directory=os.path.dirname(__file__))
         w.button_yes.clicked.connect(w.accept)
@@ -1351,8 +1353,7 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
                     others[0], others[i] = others[i], others[0]
                     break
 
-        label = others[0].label
-        w.merged_label.setText(label)
+        w.merged_label.setText(suggested_label)
 
         entries = [QtWidgets.QListWidgetItem(other.label) for other in others]
         for e in entries:
@@ -1372,6 +1373,17 @@ class GlueApplication(Application, QtWidgets.QMainWindow):
 
         return None, None
 
+    def screenshot(self, filename):
+        """
+        Save a screenshot of the current application window to a file.
+        """
+        image = QtGui.QImage(self.size(), QtGui.QImage.Format_RGB32)
+        painter = QtGui.QPainter(image)
+        flags = self.IgnoreMask | self.DrawWindowBackground | self.DrawChildren
+        self.render(painter, QtCore.QPoint(), QtGui.QRegion(), flags)
+        image.save(filename)
+        painter.end()
+
     def __gluestate__(self, context):
         state = super(GlueApplication, self).__gluestate__(context)
         state['tab_names'] = self.tab_names
