@@ -7,7 +7,7 @@ from collections import OrderedDict
 from glue.core.coordinates import coordinates_from_header, WCSCoordinates
 from glue.core.data import Component, Data
 from glue.config import data_factory, qglue_parser
-
+from glue.external.six import string_types
 
 __all__ = ['is_fits', 'fits_reader', 'is_casalike', 'casalike_cube']
 
@@ -17,7 +17,7 @@ def is_fits(filename):
     try:
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
-            with fits.open(filename, ignore_missing_end=True):
+            with fits.open(filename, ignore_missing_end=True, mode='denywrite'):
                 return True
     except IOError:
         return False
@@ -57,7 +57,7 @@ def fits_reader(source, auto_merge=False, exclude_exts=None, label=None):
         hdulist = source
         close_hdulist = False
     else:
-        hdulist = fits.open(source, ignore_missing_end=True)
+        hdulist = fits.open(source, ignore_missing_end=True, mode='denywrite')
         hdulist.verify('fix')
         close_hdulist = True
 
@@ -80,14 +80,27 @@ def fits_reader(source, auto_merge=False, exclude_exts=None, label=None):
             label_base = basename(hdulist_name)
 
     # Create a new image Data.
-    def new_data():
-        label = '{0}[{1}]'.format(
-            label_base,
-            hdu_name
-        )
+    def new_data(suffix=True):
+        if suffix:
+            label = '{0}[{1}]'.format(label_base, hdu_name)
+        else:
+            label = label_base
         data = Data(label=label)
         data.coords = coords
-        data.meta.update(OrderedDict(hdu.header))
+
+        # We need to be careful here because some header values are special
+        # objects that we should convert to strings
+        for key, value in hdu.header.items():
+            if (key == 'COMMENT' or key == 'HISTORY'):
+                if key not in data.meta:
+                    data.meta[key] = [str(value)]
+                else:
+                    data.meta[key].append(str(value))
+            elif isinstance(value, string_types) or isinstance(value, (int, float, bool)):
+                data.meta[key] = value
+            else:
+                data.meta[key] = str(value)
+
         groups[hdu_name] = data
         extension_by_shape[shape] = hdu_name
         return data
@@ -101,14 +114,16 @@ def fits_reader(source, auto_merge=False, exclude_exts=None, label=None):
             if is_image_hdu(hdu):
                 shape = hdu.data.shape
                 coords = coordinates_from_header(hdu.header)
+                units = hdu.header.get('BUNIT')
                 if not auto_merge or has_wcs(coords):
-                    data = new_data()
+                    data = new_data(suffix=len(hdulist) > 1)
                 else:
                     try:
                         data = groups[extension_by_shape[shape]]
                     except KeyError:
-                        data = new_data()
-                data.add_component(component=hdu.data,
+                        data = new_data(suffix=len(hdulist) > 1)
+                component = Component.autotyped(hdu.data, units=units)
+                data.add_component(component=component,
                                    label=hdu_name)
             elif is_table_hdu(hdu):
                 # Loop through columns and make component list
@@ -158,7 +173,7 @@ def is_casalike(filename, **kwargs):
 
     if not is_fits(filename):
         return False
-    with fits.open(filename, ignore_missing_end=True) as hdulist:
+    with fits.open(filename, ignore_missing_end=True, mode='denywrite') as hdulist:
         if len(hdulist) != 1:
             return False
         if hdulist[0].header['NAXIS'] != 4:
@@ -187,12 +202,14 @@ def casalike_cube(filename, **kwargs):
     if 'ignore_missing_end' not in kwargs:
         kwargs['ignore_missing_end'] = True
 
-    with fits.open(filename, **kwargs) as hdulist:
+    with fits.open(filename, mode='denywrite', **kwargs) as hdulist:
         array = hdulist[0].data
         header = hdulist[0].header
     result.coords = coordinates_from_header(header)
     for i in range(array.shape[0]):
-        result.add_component(array[[i]], label='STOKES %i' % i)
+        units = header.get('BUNIT')
+        component = Component.autotyped(array[[i]], units=units)
+        result.add_component(component, label='STOKES %i' % i)
     return result
 
 
