@@ -6,7 +6,7 @@ import numpy as np
 
 from qtpy import QtCore, QtWidgets
 from glue.utils.qt import load_ui
-from glue.utils import nonpartial
+from glue.utils import nonpartial, format_minimal
 from glue.icons.qt import get_icon
 from glue.core.state_objects import State, CallbackProperty
 from glue.external.echo.qt import autoconnect_callbacks_to_qt
@@ -67,8 +67,17 @@ class SliceWidget(QtWidgets.QWidget):
         self.value_slice_center.setMaximum(hi)
         self.value_slice_center.valueChanged.connect(nonpartial(self.set_label_from_slider))
 
+        # Figure out the optimal format to use to show the world values. We do
+        # this by figuring out the precision needed so that when converted to
+        # a string, every string value is different.
+
+        if world is not None and len(world) > 1:
+            self.label_fmt = format_minimal(world)[0]
+        else:
+            self.label_fmt = "{:g}"
+
         self.text_slider_label.setMinimumWidth(80)
-        self.state.slider_label = str(self.value_slice_center.value())
+        self.state.slider_label = self.label_fmt.format(self.value_slice_center.value())
         self.text_slider_label.editingFinished.connect(nonpartial(self.set_slider_from_label))
 
         self._play_timer = QtCore.QTimer()
@@ -109,10 +118,11 @@ class SliceWidget(QtWidgets.QWidget):
             else:
                 self.text_warning.hide()
             self.state.slider_unit = self._world_unit
+            self.state.slider_label = self.label_fmt.format(value)
         else:
             self.text_warning.hide()
             self.state.slider_unit = ''
-        self.state.slider_label = str(value)
+            self.state.slider_label = str(value)
 
     def set_slider_from_label(self):
 
@@ -128,7 +138,7 @@ class SliceWidget(QtWidgets.QWidget):
         if self.state.use_world:
             # Don't want to assume world is sorted, pick closest value
             value = np.argmin(np.abs(self._world - float(text)))
-            self.state.slider_label = str(self._world[value])
+            self.state.slider_label = self.label_fmt.format(self._world[value])
         else:
             value = int(text)
         self.value_slice_center.setValue(value)
